/* ----------------------------------------------------------------- */
/*                                                                   */
/*  Copyright (c) 2009-2011  Nagoya Institute of Technology          */
/*                           Department of Computer Science          */
/*                2010-2012  hkrn                                    */
/*                                                                   */
/* All rights reserved.                                              */
/*                                                                   */
/* Redistribution and use in source and binary forms, with or        */
/* without modification, are permitted provided that the following   */
/* conditions are met:                                               */
/*                                                                   */
/* - Redistributions of source code must retain the above copyright  */
/*   notice, this list of conditions and the following disclaimer.   */
/* - Redistributions in binary form must reproduce the above         */
/*   copyright notice, this list of conditions and the following     */
/*   disclaimer in the documentation and/or other materials provided */
/*   with the distribution.                                          */
/* - Neither the name of the MMDAI project team nor the names of     */
/*   its contributors may be used to endorse or promote products     */
/*   derived from this software without specific prior written       */
/*   permission.                                                     */
/*                                                                   */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND            */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,       */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF          */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE          */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS */
/* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,          */
/* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED   */
/* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     */
/* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON */
/* ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,   */
/* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY    */
/* OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE           */
/* POSSIBILITY OF SUCH DAMAGE.                                       */
/* ----------------------------------------------------------------- */

#ifndef TILEDSTAGE_H
#define TILEDSTAGE_H

#include <QtCore/QString>
#include <QtGui/QMatrix4x4>
#include <QtOpenGL/QGLFunctions>
#include <LinearMath/btVector3.h>
#include <vpvl2/Scene.h>

namespace vpvl2 {
class Scene;
}

namespace internal {
class Delegate;
class World;
}

class btRigidBody;

class TiledStage
{
public:
    class PrivateContext;

    TiledStage(internal::World *world);
    ~TiledStage();

    void loadFloor(const QString &path);
    void loadBackground(const QString &path);
    void renderFloor(const vpvl2::Scene *scene, const QMatrix4x4 &projection);
    void renderBackground(const vpvl2::Scene *scene, const QMatrix4x4 &projection);
    void setSize(float width, float height, float depth);

private:
    void buildFloor(float width, float height);
    void destroyFloor();

    PrivateContext *m_floor;
    PrivateContext *m_background;
    btRigidBody *m_floorRigidBody;
    internal::World *m_world;

    Q_DISABLE_COPY(TiledStage)
};

#endif // TILEDSTAGE_H
